#!/usr/bin/python3

import moiety_modeling
import itertools
import numpy as np
import jsonpickle


def createData(molecule, moietyStates, std, threshold, norm, name, path):
    """To create simulated data based on moiety states, standard deviation (std), and the threshold.

    :param molecule: the molecule.
    :param moietyStates: the list of moiety state values. [[(13C_0.18O_1, 0.1), (13C_6.18O_5, 09)], [...], [...], [...]] (according to certain order)
    :param std: the standard deviation of errors.
    :param threshold: the threshold of the detected signal.
    :param norm: to norm the dataset to make sum to 1.
    :param name: the name of the dataset.
    :param path: the path to save the datasets.
    :return: the dataset object.
    """

    # To calculate the isotopologue.
    isotopologues = {isotopologue: 0 for isotopologue in molecule.allStates}
    isotopes = sorted(molecule.moieties[0].maxIsotopeNum)
    for state in itertools.product(*moietyStates):
        temIsotopeSum = {isotope: 0 for isotope in isotopes}
        value = 1
        for moietyStates in state:
            value *= moietyStates[1]
            seperateMoiety = [seperateState.split('_') for seperateState in moietyStates[0].split('.')]
            for seperateState in seperateMoiety:
                temIsotopeSum[seperateState[0]] += int(seperateState[1])
        isotopologue = '.'.join(['{0}_{1}'.format(isotope, str(temIsotopeSum[isotope])) for isotope in isotopes])
        isotopologues[isotopologue] += value

    # To add errors.
    if std > 0:
        n = len(isotopologues)
        noise = np.random.normal(0, std, n)
        i = 0
        for isotopologue in isotopologues:
            isotopologues[isotopologue] += noise[i]
            i += 1

    # To filter the isotopologue value that is below certain threshold.
    sum = 0
    if threshold > 0:
        for isotopologue in isotopologues:
            if isotopologues[isotopologue] < threshold:
                isotopologues[isotopologue] = 0
                sum += isotopologues[isotopologue]

    # Normalize the data, let the sum be 1.
    if norm:
        sum = dictSum(isotopologues)
        isotopologues = {isotopologue: isotopologues[isotopologue] / sum for isotopologue in isotopologues}

    # to create the dataset object.
    data = []
    for isotopologue in isotopologues:
        data.append({'labelingIsotopes': isotopologue, 'height': float(isotopologues[isotopologue]), 'heightSE': 0})

    dataset = moiety_modeling.Dataset(name, {molecule.name: data})

    # save the datasets
    jsonpickle.set_encoder_options('json', sort_keys=True, indent=4)
    with open('{0}/{1}_{2}_{3}.json'.format(path, molecule.name, name, std), 'w') as outFile:
        outFile.write(jsonpickle.encode({'datasets': [dataset]}))

    return sum


def dictSum(dic):
    """
    To calculate the sum of the dict.
    :param dic: a dictionary
    :return: the sum value.
    """
    sum = 0
    for item in dic:
        sum += dic[item]
    return sum


if __name__ == "__main__":

    # create single datasets

    ribose1 = moiety_modeling.Moiety("ribose", {'13C': 5}, isotopeStates={'13C': [0, 5]}, nickname='r')
    glucose1 = moiety_modeling.Moiety("glucose", {'13C': 6}, isotopeStates={'13C': [0, 6]}, nickname='g')
    acetyl1 = moiety_modeling.Moiety("acetyl", {'13C': 2}, isotopeStates={'13C': [0, 2]}, nickname='a')
    uracil1 = moiety_modeling.Moiety("uracil", {'13C': 4}, isotopeStates={'13C': [0, 1, 2, 3]}, nickname='u')

    UDP_GlcNAC1 = moiety_modeling.Molecule('UDP_GlcNAC', [ribose1, glucose1, acetyl1, uracil1])

    moietyStates = [ [('13C_0', 0.1), ('13C_5', 0.9)], [('13C_0', 0.1), ('13C_6', 0.9)], [('13C_0', 0.7), ('13C_2', 0.3)], [('13C_0', 0.2), ('13C_1', 0.2), ('13C_2', 0.5), ('13C_3', 0.1)] ]

    # # std = 0.1
    path_single_1 = '/mlab/data/hji236/projects/moiety_modeling_analyses/results/single_std_0.1_dataset'
    for i in range(100):
        createData(UDP_GlcNAC1, moietyStates, 0.1, 0.005, True, 'single_{0}'.format(i), path_single_1)

    # # std = 0.01
    path_single_2 = '/mlab/data/hji236/projects/moiety_modeling_analyses/results/single_std_0.01_dataset'
    for i in range(100):
        createData(UDP_GlcNAC1, moietyStates, 0.01, 0.005, True, 'single_{0}'.format(i), path_single_2)

    # # std = 0.001
    path_single_3 = '/mlab/data/hji236/projects/moiety_modeling_analyses/results/single_std_0.001_dataset'
    for i in range(100):
        createData(UDP_GlcNAC1, moietyStates, 0.001, 0.005, True, 'single_{0}'.format(i), path_single_3)


    # create multi trace datasets.
    ribose2 = moiety_modeling.Moiety("ribose", {'13C': 5, '18O': 4}, states=['13C_0.18O_0', '13C_5.18O_4'], nickname='r')
    glucose2 = moiety_modeling.Moiety("glucose", {'13C': 6, '18O': 5}, states=['13C_0.18O_0', '13C_6.18O_5'], nickname='g')
    acetyl2 = moiety_modeling.Moiety("acetyl", {'13C': 2, '18O': 1}, states=['13C_0.18O_0', '13C_2.18O_1'], nickname='a')
    uracil2 = moiety_modeling.Moiety("uracil", {'13C': 4, '18O': 2}, states=['13C_0.18O_0', '13C_1.18O_0', '13C_2.18O_1', '13C_2.18O_0', '13C_3.18O_0', '13C_3.18O_1'], nickname='u')

    UDP_GlcNAC2 = moiety_modeling.Molecule('UDP_GlcNAC', [ribose2, glucose2, acetyl2, uracil2])

    moietyStates = [[('13C_0.18O_0', 0.1), ('13C_5.18O_4', 0.9)], [('13C_0.18O_0', 0.1), ('13C_6.18O_5', 0.9)], [('13C_0.18O_0', 0.7), ('13C_2.18O_1', 0.3)], [('13C_0.18O_0', 0.2), ('13C_1.18O_0', 0.2), ('13C_2.18O_1', 0.25), ('13C_2.18O_0', 0.25), ('13C_3.18O_0', 0.05), ('13C_3.18O_1', 0.05)]]

    # # std = 0.1
    path_multi_1 = '/mlab/data/hji236/projects/moiety_modeling_analyses/results/multi_std_0.1_dataset'
    for i in range(100):
        createData(UDP_GlcNAC2, moietyStates, 0.1, 0.005, True, 'multi_{0}'.format(i), path_multi_1)

    # std = 0.01
    path_multi_2 = '/mlab/data/hji236/projects/moiety_modeling_analyses/results/multi_std_0.01_dataset'
    for i in range(100):
        createData(UDP_GlcNAC2, moietyStates, 0.01, 0.005, True, 'multi_{0}'.format(i), path_multi_2)

    # # std = 0.001
    path_multi_3 = '/mlab/data/hji236/projects/moiety_modeling_analyses/results/multi_std_0.001_dataset'
    for i in range(100):
        createData(UDP_GlcNAC2, moietyStates, 0.001, 0.005, True, 'multi_{0}'.format(i), path_multi_3)


